<?php
/**
 * This file contains all the functions used and performed for this plugin.
 * 
 * @since 1.0
 */

/**
 * Creates the blogs.dir directory in the wp-content folder
 * 
 * @since 1.4
 * @return null on success, error message on failure
 */
function multi_site_create_blogs_dir () {
	$error_message = '<div class="error"><p>WARNING: Could not create folder /wp-content/blogs.dir/. Please create this folder youself.</p></div>';
	
	if ( check_for_blogs_dir() ) {
		return null;
	}
	
	$did_it_work = @mkdir ( WORDPRESS_ROOT_DIR . '/wp-content/blogs.dir' );
	
	if ( ! $did_it_work ) {
		return $error_message;
	} else {
		return null;
	}
}

/**
 * Checks if the currently installed version of the plugin has been updated.
 * If so then it sends out an upgrade reqest to the site admin.
 * 
 * If the upgrade request is made then it loads the upgrade.inc.php file
 * and peforms the upgrade task.
 * 
 * @since 1.4
 * @return bool If it worked or not
 */
function enable_multi_site_update_patcher () {
	if ( ! isset( $_GET ['enable-multi-site-peform-upgrade'] ) ) {
		// Show alert box
		add_action ( 'admin_notices', 'enable_multi_site_update_alert' );
	} else {
		// Perform needed updates
		include MULTI_SITE_INC_DIR . '/upgrade.inc.php';
	}	
}
/**
 * Alert box that displays when an upgrade needs to be peformed
 * Echos text
 * 
 * @since 1.4
 * @return null
 */
function enable_multi_site_update_alert () {
	?>
	<div class="updated">
		<p style="text-align:center;">
			You have recently updated your Multi-Site Plugin. It needs to perform a patch. 
			<a href="options-general.php?page=enable-multi-site-plugin&enable-multi-site-peform-upgrade=<?php echo EMS_PLUGIN_VERSION; ?>">Click here</a> to apply the patch.
		</p>
	</div>
	<?php
}

/** 
 * Check if the wp-content folder is already made for the blogs.dir directory
 * 
 * @since 1.4
 * @return bool Whether the folder is already made
 */
function check_for_blogs_dir() {
	if ( is_dir( WORDPRESS_ROOT_DIR . '/wp-content/blogs.dir' ) ) {
		return true;
	} else {
		return false;
	}
}

/**
 * Check if the .htaccss file exist and is editable
 * 
 * @since 1.0
 * @return bool Whether the file exist and is writeable
 */
function check_for_htaccess() {
	if ( file_exists( WORDPRESS_ROOT_DIR . '/.htaccess' ) ) {
		if (is_writable( WORDPRESS_ROOT_DIR . '/.htaccess' ) ) {
			return true;
		} else {
			return false;
		}
	} elseif ( is_writable( WORDPRESS_ROOT_DIR ) ) {
		return true;
	} else {
		return false;
	}
}

/**
 * Check if the wp-config.php file exist and is editable
 * 
 * @since 1.0
 * @return bool Whether the file exist and is writable
 */
function check_for_wp_config() {
if ( file_exists( WORDPRESS_ROOT_DIR . '/wp-config.php' ) && is_writable( WORDPRESS_ROOT_DIR . '/wp-config.php' ) ) {
		return true;
	} else {
		return false;
	}
}

/**
 * Check for an existing network.
 *
 * @since 1.0.0
 * @return Whether a network exists.
 */
function enable_multi_site_network_domain_check() {
	global $wpdb;
	if ( $wpdb->get_var( "SHOW TABLES LIKE '$wpdb->site'" ) )
		return $wpdb->get_var( "SELECT domain FROM $wpdb->site ORDER BY id ASC LIMIT 1" );
	return false;
}

/**
 * Allow subdomain install
 *
 * @since 1.0.0
 * @return bool Whether subdomain install is allowed
 */
function enable_multi_site_allow_subdomain_install() {
	$domain = preg_replace( '|https?://([^/]+)|', '$1', get_option( 'siteurl' ) );
	if( false !== strpos( $domain, '/' ) || 'localhost' == $domain || preg_match( '|[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+|', $domain ) )
		return false;

	return true;
}
/**
 * Allow subdirectory install
 *
 * @since 1.0.0
 * @return bool Whether subdirectory install is allowed
 */
function enable_multi_site_allow_subdirectory_install() {
	global $wpdb;
	if ( apply_filters( 'allow_subdirectory_install', false ) )
		return true;

	if ( defined( 'ALLOW_SUBDIRECTORY_INSTALL' ) && ALLOW_SUBDIRECTORY_INSTALL )
		return true;

	$post = $wpdb->get_row( "SELECT ID FROM $wpdb->posts WHERE post_date < DATE_SUB(NOW(), INTERVAL 1 MONTH) AND post_status = 'publish'" );
	if ( empty( $post ) )
		return true;

	return false;
}
/**
 * Get base domain of network.
 *
 * @since 1.0.0
 * @return string Base domain.
 */
function enable_multi_site_get_clean_basedomain() {
	if ( $existing_domain = enable_multi_site_network_domain_check() )
		return $existing_domain;
	$domain = preg_replace( '|https?://|', '', get_option( 'siteurl' ) );
	if ( $slash = strpos( $domain, '/' ) )
		$domain = substr( $domain, 0, $slash );
	return $domain;
}

/**
 * Prints step 1 for Network installation process.
 *
 * @since 1.0.0
 */
function enable_multi_site_network_step1( $errors = false ) {
	global $is_apache;

	if ( get_option( 'siteurl' ) != get_option( 'home' ) ) {
		echo '<div class="error"><p><strong>' . __('Error:') . '</strong> ' . sprintf( __( 'Your <strong>WordPress address</strong> must match your <strong>Site address</strong> before creating a Network. See <a href="%s">General Settings</a>.' ), esc_url( admin_url( 'options-general.php' ) ) ) . '</p></div>';
		echo '</div>';
		include MULTI_SITE_INC_DIR . '/page-footer.php';
		die();
	}

	$hostname = enable_multi_site_get_clean_basedomain();
	$has_ports = strstr( $hostname, ':' );
	if ( ( false !== $has_ports && ! in_array( $has_ports, array( ':80', ':443' ) ) ) ) {
		echo '<div class="error"><p><strong>' . __( 'Error:') . '</strong> ' . __( 'You cannot install a network of sites with your server address.' ) . '</p></div>';
		echo '<p>' . sprintf( __( 'You cannot use port numbers such as <code>%s</code>.' ), $has_ports ) . '</p>';
		echo '<a href="' . esc_url( admin_url() ) . '">' . __( 'Return to Dashboard' ) . '</a>';
		echo '</div>';
		include MULTI_SITE_INC_DIR . '/page-footer.php';
		die();
	}

	echo '<form method="post" action="">';

	wp_nonce_field( 'plugin-install-network-1' );

	$error_codes = array();
	if ( is_wp_error( $errors ) ) {
		echo '<div class="error"><p><strong>' . __( 'ERROR: The network could not be created.' ) . '</strong></p>';
		foreach ( $errors->get_error_messages() as $error )
			echo "<p>$error</p>";
		echo '</div>';
		$error_codes = $errors->get_error_codes();
	}

	if ( WP_CONTENT_DIR != ABSPATH . 'wp-content' )
		echo '<div class="error"><p><strong>' . __('Warning!') . '</strong> ' . __( 'Networks may not be fully compatible with custom wp-content directories.' ) . '</p></div>';

	$site_name = ( ! empty( $_POST['sitename'] ) && ! in_array( 'empty_sitename', $error_codes ) ) ? $_POST['sitename'] : sprintf( _x('%s Sites', 'Default network name' ), get_option( 'blogname' ) );
	$admin_email = ( ! empty( $_POST['email'] ) && ! in_array( 'invalid_email', $error_codes ) ) ? $_POST['email'] : get_option( 'admin_email' );
	?>
	<p><?php _e( 'Welcome to the Network installation process!' ); ?></p>
	<p><?php _e( 'Fill in the information below and you&#8217;ll be on your way to creating a network of WordPress sites. We will create configuration files for you.' ); ?></p>
	<?php

	if ( isset( $_POST['subdomain_install'] ) ) {
		$subdomain_install = (bool) $_POST['subdomain_install'];
	} elseif ( apache_mod_loaded('mod_rewrite') ) { // assume nothing
		$subdomain_install = true;
	} elseif ( !enable_multi_site_allow_subdirectory_install() ) {
		$subdomain_install = true;
	} else {
		$subdomain_install = false;
		if ( $got_mod_rewrite = got_mod_rewrite() ) // dangerous assumptions
			echo '<div class="updated inline"><p><strong>' . __( 'Note:' ) . '</strong> ' . __( 'Please make sure the Apache <code>mod_rewrite</code> module is installed as it will be used at the end of this installation.' ) . '</p>';
		elseif ( $is_apache )
			echo '<div class="error inline"><p><strong>' . __( 'Warning!' ) . '</strong> ' . __( 'It looks like the Apache <code>mod_rewrite</code> module is not installed.' ) . '</p>';
		if ( $got_mod_rewrite || $is_apache ) // Protect against mod_rewrite mimicry (but ! Apache)
			echo '<p>' . __( 'If <code>mod_rewrite</code> is disabled, ask your administrator to enable that module, or look at the <a href="http://httpd.apache.org/docs/mod/mod_rewrite.html">Apache documentation</a> or <a href="http://www.google.com/search?q=apache+mod_rewrite">elsewhere</a> for help setting it up.' ) . '</p></div>';
	}

	if ( enable_multi_site_allow_subdomain_install() && enable_multi_site_allow_subdirectory_install() ) : ?>
		<h3><?php esc_html_e( 'Addresses of Sites in your Network' ); ?></h3>
		<p><?php _e( 'Please choose whether you would like sites in your WordPress network to use sub-domains or sub-directories. <strong>You cannot change this later.</strong>' ); ?></p>
		<p><?php _e( 'You will need a wildcard DNS record if you are going to use the virtual host (sub-domain) functionality.' ); ?></p>
		<?php // @todo: Link to an MS readme? ?>
		<table class="form-table">
			<tr>
				<th><label><input type='radio' name='subdomain_install' value='1'<?php checked( $subdomain_install ); ?> /> <?php _e( 'Sub-domains' ); ?></label></th>
				<td><?php printf( _x( 'like <code>site1.%1$s</code> and <code>site2.%1$s</code>', 'subdomain examples' ), $hostname ); ?></td>
			</tr>
			<tr>
				<th><label><input type='radio' name='subdomain_install' value='0'<?php checked( ! $subdomain_install ); ?> /> <?php _e( 'Sub-directories' ); ?></label></th>
				<td><?php printf( _x( 'like <code>%1$s/site1</code> and <code>%1$s/site2</code>', 'subdirectory examples' ), $hostname ); ?></td>
			</tr>
		</table>

<?php
	endif;

		$is_www = ( 0 === strpos( $hostname, 'www.' ) );
		if ( $is_www ) :
		?>
		<h3><?php esc_html_e( 'Server Address' ); ?></h3>
		<p><?php printf( __( 'We recommend you change your siteurl to <code>%1$s</code> before enabling the network feature. It will still be possible to visit your site using the <code>www</code> prefix with an address like <code>%2$s</code> but any links will not have the <code>www</code> prefix.' ), substr( $hostname, 4 ), $hostname ); ?></h3>
		<table class="form-table">
			<tr>
				<th scope='row'><?php esc_html_e( 'Server Address' ); ?></th>
				<td>
					<?php printf( __( 'The internet address of your network will be <code>%s</code>.' ), $hostname ); ?>
				</td>
			</tr>
		</table>
		<?php endif; ?>

		<h3><?php esc_html_e( 'Network Details' ); ?></h3>
		<table class="form-table">
		<?php if ( 'localhost' == $hostname ) : ?>
			<tr>
				<th scope="row"><?php esc_html_e( 'Sub-directory Install' ); ?></th>
				<td><?php
					_e( 'Because you are using <code>localhost</code>, the sites in your WordPress network must use sub-directories. Consider using <code>localhost.localdomain</code> if you wish to use sub-domains.' );
					// Uh oh:
					if ( !enable_multi_site_allow_subdirectory_install() )
						echo ' <strong>' . __( 'Warning!' ) . ' ' . __( 'The main site in a sub-directory install will need to use a modified permalink structure, potentially breaking existing links.' ) . '</strong>';
				?></td>
			</tr>
		<?php elseif ( !enable_multi_site_allow_subdomain_install() ) : ?>
			<tr>
				<th scope="row"><?php esc_html_e( 'Sub-directory Install' ); ?></th>
				<td><?php
					_e( 'Because your install is in a directory, the sites in your WordPress network must use sub-directories.' );
					// Uh oh:
					if ( !enable_multi_site_allow_subdirectory_install() )
						echo ' <strong>' . __( 'Warning!' ) . ' ' . __( 'The main site in a sub-directory install will need to use a modified permalink structure, potentially breaking existing links.' ) . '</strong>';
				?></td>
			</tr>
		<?php elseif ( !enable_multi_site_allow_subdirectory_install() ) : ?>
			<tr>
				<th scope="row"><?php esc_html_e( 'Sub-domain Install' ); ?></th>
				<td><?php _e( 'Because your install is not new, the sites in your WordPress network must use sub-domains.' );
					echo ' <strong>' . __( 'The main site in a sub-directory install will need to use a modified permalink structure, potentially breaking existing links.' ) . '</strong>';
				?></td>
			</tr>
		<?php endif; ?>
		<?php if ( ! $is_www ) : ?>
			<tr>
				<th scope='row'><?php esc_html_e( 'Server Address' ); ?></th>
				<td>
					<?php printf( __( 'The internet address of your network will be <code>%s</code>.' ), $hostname ); ?>
				</td>
			</tr>
		<?php endif; ?>
			<tr>
				<th scope='row'><?php esc_html_e( 'Network Title' ); ?></th>
				<td>
					<input name='sitename' type='text' size='45' value='<?php echo esc_attr( $site_name ); ?>' />
					<br /><?php _e( 'What would you like to call your network?' ); ?>
				</td>
			</tr>
			<tr>
				<th scope='row'><?php esc_html_e( 'Admin E-mail Address' ); ?></th>
				<td>
					<input name='email' type='text' size='45' value='<?php echo esc_attr( $admin_email ); ?>' />
					<br /><?php _e( 'Your email address.' ); ?>
				</td>
			</tr>
		</table>
		<p class='submit'><input class="button-primary" name='submit' type='submit' value='<?php esc_attr_e( 'Install' ); ?>' /></p>
	</form>
		<?php
}

/**
 * Step 2 functions
 *
 * @since 1.0.0
 * @return null
 */
function enable_multi_site_network_step2 ( $errors = false ) {
	global $base, $wpdb;
	$hostname = enable_multi_site_get_clean_basedomain ();
	
	// Wildcard DNS message.
	if ( is_wp_error ( $errors ) ) echo '<div class="error">' . $errors->get_error_message () . '</div>';
	
	if ( $_POST ) {
		$subdomain_install = enable_multi_site_allow_subdomain_install () ? (enable_multi_site_allow_subdirectory_install () ? ! empty ( $_POST ['subdomain_install'] ) : true) : false;
	} else {
		if ( is_multisite () ) {
			$subdomain_install = is_subdomain_install ();
		} else {
			$subdomain_install = ( bool ) $wpdb->get_var ( "SELECT meta_value FROM $wpdb->sitemeta WHERE site_id = 1 AND meta_key = 'subdomain_install'" );
		}
	}
	
	// Creates config file additions
	if ( $_POST || ! is_multisite () ) {
		
		// Gets the current config data
		$config_file_contents = file_get_contents ( WORDPRESS_ROOT_DIR . '/wp-config.php' );
		
		// Changes the bool of if subdomain will be installed into text for use in the config file
		$subdomain_install = ( $subdomain_install ) ? 'true' : 'false';
		
		$config_file_stuff = "define( 'MULTISITE', true );\n";
		$config_file_stuff .= "define( 'SUBDOMAIN_INSTALL', " . $subdomain_install . " );\n";
		$config_file_stuff .= '$base = "' . $base . '";' . "\n";
		$config_file_stuff .= "define( 'DOMAIN_CURRENT_SITE', '" . $hostname . "' );\n";
		$config_file_stuff .= "define( 'PATH_CURRENT_SITE', '" .  $base . "' );\n";
		$config_file_stuff .= "define( 'SITE_ID_CURRENT_SITE', 1 );\n";
		$config_file_stuff .= "define( 'BLOG_ID_CURRENT_SITE', 1 );\n";
		$config_file_stuff .= '/* This shit was made multi-site enabled by @link http://jgwebdevelopment.com */' . "\n";
		
		// Adds a friendly message and adds new defines
		$updated_config_file_contents = str_ireplace ( '/* That\'s all, stop editing! Happy blogging. */', $config_file_stuff, $config_file_contents );
		
		// Saves the changes
		file_put_contents(WORDPRESS_ROOT_DIR . '/wp-config.php', $updated_config_file_contents);
	}
	
	// Fix back to bool from string conversion above. Fixes Bug in v1.3
	$subdomain_install = ( $subdomain_install == 'true' || $subdomain_install == true ) ? true : false;
	
	// Creates htaccess file
	$htaccess_file = '#Made Multi-Site with Multi-Site Enabler ' . EMS_PLUGIN_VERSION . '#' . "\n";
	$htaccess_file .= 'RewriteEngine On' . "\n";
	$htaccess_file .= 'RewriteBase ' . $base . "\n";
	$htaccess_file .= 'RewriteRule ^index\.php$ - [L]' . "\n";
	$htaccess_file .= "\n";
	$htaccess_file .= '# uploaded files' . "\n";
	$htaccess_file .= 'RewriteRule ^' . ($subdomain_install ? '' : '([_0-9a-zA-Z-]+/)?') . 'files/(.+) wp-includes/ms-files.php?file=$' . ($subdomain_install ? 1 : 2) . ' [L]' . "\n";
	
	if ( ! $subdomain_install ) $htaccess_file .= "\n# add a trailing slash to /wp-admin\n" . 'RewriteRule ^([_0-9a-zA-Z-]+/)?wp-admin$ $1wp-admin/ [R=301,L]' . "\n";
	
	$htaccess_file .= "\r\n" . 'RewriteCond %{REQUEST_FILENAME} -f [OR]' . "\n";
	$htaccess_file .= 'RewriteCond %{REQUEST_FILENAME} -d' . "\n";
	$htaccess_file .= 'RewriteRule ^ - [L]' . "\n";
	
	if ( ! $subdomain_install ) $htaccess_file .= "\nRewriteRule  ^([_0-9a-zA-Z-]+/)?(wp-(content|admin|includes).*) $2 [L]\nRewriteRule  ^([_0-9a-zA-Z-]+/)?(.*\.php)$ $2 [L]";
	
	$htaccess_file .= "\nRewriteRule . index.php [L]";
	
	// Saves the file like Jesus
	if ( check_for_htaccess () ) {
		$myFile = WORDPRESS_ROOT_DIR . '/.htaccess';
		$fh = fopen ( $myFile, 'w' );
		fwrite ( $fh, $htaccess_file );
		fclose ( $fh );
	} else {
		echo '<div class="error"><p>The htaccess file was not updated correctly!</p></div>';
	}
	
	// Creates blogs.dir directory	
	echo multi_site_create_blogs_dir();
	
	// Sends out success message.
	echo '<h3>You will now need to <a href="' . get_bloginfo('url') . '/wp-login.php">login again.</a></h3>';
	
	// Includes new home page of promotional info
	include MULTI_SITE_INC_DIR . '/give-me-money.inc.php';
}


?>